<?php

/**
 * @file
 * Admin settings include for dotmailer module
 */

/**
 * Form callback for the admin settings for dotmailer
 */
function dotmailer_admin_settings(&$form, &$form_state) {

  $form['dotmailer_account_info'] = array(
    '#type' => 'fieldset',
    '#title' => 'Dotmailer Account Information',
  );

  // Only show the book selection forms if account information has been provided
  $username = variable_get('dotmailer_username');
  $password = variable_get('dotmailer_password');

  if ($username && $password) {
    $form = _dotmailer_admin_settings_with_account($form, $form_state);
  }
  else{
    $form = _dotmailer_admin_settings_no_account($form, $form_state);
  }

  return system_settings_form($form);
}

/**
 * Helper for providing the form elements when an account is active
 */
function _dotmailer_admin_settings_with_account(&$form, &$form_state) {

  $username = variable_get('dotmailer_username');
  $form['dotmailer_account_info']['dotmailer_setup_account'] = array(
    '#markup' => '<p>' . t('Current configured account is for: @username', array('@username' => $username)) . '</p>',
  );

  $form['dotmailer_account_info']['reset'] = array(
    '#markup' => '<p>' . l('Change these details.', 'admin/config/dotmailer/reset') . '</p>',
  );


  $address_books = dotmailer_listaddressbooks();

  if ($address_books) {
    $available_lists = array();

    $form['dotmailer_books'] = array(
      '#type' => 'fieldset',
      '#collapsible' => FALSE,
      '#title' => 'dotmailer Subscription Lists',
      '#tree' => TRUE
    );

    $active_lists = variable_get('dotmailer_active_lists');

    if (!$active_lists) {
      $form['dotmailer_books']['none_available']['#markup'] = t('Currently no lists are active.');
    }

    foreach ($address_books as $address_book) {

      $available_lists[$address_book->ID] = "$address_book->Name [$address_book->ID]";

      if ($active_lists && in_array($address_book->ID, $active_lists)) {
        // Get the stored books
        $form['dotmailer_books']['dotmailer_book_' . $address_book->ID] = _dotmailer_admin_settings_address_book_settings($address_book);
      }
    }

    // Active list configuration
    $form['dotmailer_active_books'] = array(
      '#type' => 'fieldset',
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#weight' => 0,
      '#title' => 'Active Address books',
    );

    $form['dotmailer_active_books']['dotmailer_active_lists'] = array(
      '#type' => 'select',
      '#options' => $available_lists,
      '#default_value' => $active_lists,
      '#multiple' => 'multiple',
      '#description' => t('Select the dotMailer address books to use within your Drupal site.'),
    );

  }
  else{
    drupal_set_message(t('You do not have any valid dotmailer address books.'), 'error');
    drupal_set_message($address_books, 'error'); // Books is a string if there is an error.
  }


  // Messages configuration
  $form['dotmailer_messages'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#title' => 'Anonymous Lists Messaging',
  );
  $form['dotmailer_messages']['dotmailer_subscription_success_message'] = array(
    '#type' => 'textarea',
    '#title' => t('Subscription Success Message'),
    '#default_value' => variable_get('dotmailer_subscription_success_message', t('Thank you, you have been successfully subscribed.'))
  );
  $form['dotmailer_messages']['dotmailer_subscription_failure_message'] = array(
    '#type' => 'textarea',
    '#title' => t('Subscription Failure Message'),
    '#default_value' => variable_get('dotmailer_subscription_failure_message', t('We were unable to subscribe you at this time. Please try again later.'))
  );
  $form['dotmailer_messages']['dotmailer_unsubscription_success_message'] = array(
    '#type' => 'textarea',
    '#title' => t('Unsubscription Success Message'),
    '#default_value' => variable_get('dotmailer_unsubscription_success_message', t('Thank you, you have been successfully unsubscribed.'))
  );
  $form['dotmailer_messages']['dotmailer_unsubscription_failure_message'] = array(
    '#type' => 'textarea',
    '#title' => t('Unsubscription Failure Message'),
    '#default_value' => variable_get('dotmailer_unsubscription_failure_message', t('We were unable to unsubscribe you at this time. Please try again later.'))
  );

  // Extra settings
  $form['dotmailer_user_register'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show subscription options on the user registration form.'),
    '#description' => t('This will only apply for books granted to the authenticated role.'),
    '#default_value' => variable_get('dotmailer_user_register', FALSE)
  );
  $form['dotmailer_user_edit'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show Subscription Options on User Edit Screen'),
    '#description' => t('If set, a tab will be presented for managing newsletter subscriptions when editing an account.'),
    '#default_value' => variable_get('dotmailer_user_edit', FALSE)
  );

  return $form;
}

/**
 * Provides the form elements for the settings for each address book.
 */
function _dotmailer_admin_settings_address_book_settings($address_book) {
  static $weight = 0;

  $saved_book = dotmailer_get_saved_book($address_book);


  $element =  array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#title' => $address_book->Name,
    '#tree' => TRUE,
  );

  $element['book_id'] = array(
    '#type' => 'value',
    '#value' => $address_book->ID,
  );

  $element['name'] = array(
    '#type' => 'value',
    '#value' => $address_book->Name,
  );

  // Few fields for use with sign up block
  // Title
  $element['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Display title for address book'),
    '#default_value' => $saved_book->title,
    '#maxlength' => 127
  );

  // List weight
  $element['weight'] = array(
    '#type' => 'weight',
    '#title' => t('Weight'),
    '#default_value' => $saved_book->weight ? $saved_book->weight : $weight++,
    '#delta' => 20,
    '#description' => t('Weight of the book in the list, if same as another it will be sorted alphabetically')
  );

  // URL for external viewing information
  $element['external_url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL to view information about this list'),
    '#default_value' => $saved_book->external_url,
    '#description' => t('Used as a URL field so remember to include http://'),
    '#maxlength' => 127
  );

  // Add a row of checkboxes for role enabling.
  $element['roles'] = array(
    '#type' => 'fieldset',
    '#title' => t('Roles'),
    '#description' => t('Choose which roles may subscribe to this book. All users will see the books they\'re eligble for at the !subscribe and in the subscription block. Lists assigned to the Authenticated role may also apear in the registration form if that option is selected below. Authenticated user may also manage their book settings from their profile.', array('!subscribe' => l(t('newsletter subscription page'), 'dotmailer/subscribe'))),
    '#tree' => TRUE,
  );

  // Loop through the roles and allow the admin to assign a book to a role
  foreach (user_roles() as $rid => $name) {
    $element['roles'][$rid] = array(
      '#type' => 'checkbox',
      '#title' => $name,
      '#default_value' => isset($saved_book->roles[$rid])?$saved_book->roles[$rid]:FALSE,
    );
  }

  $element['description'] = array(
    '#type' => 'textarea',
    '#title' => t('List Description'),
    '#default_value' => $saved_book->description,
    '#description' => t('This description will be shown to the user on the book signup and user account settings pages')
  );

  $element['booktype'] = array(
    '#type' => 'select',
    '#title' => t('Subscription Method'),
    '#options' => array(DOTMAILER_BOOKTYPE_OPTIN => "Opt-in", DOTMAILER_BOOKTYPE_OPTOUT => 'Opt-out', DOTMAILER_BOOKTYPE_REQUIRED => 'Required'),
    '#default_value' => ($saved_book->booktype) ? $saved_book->booktype : DOTMAILER_BOOKTYPE_OPTOUT,
    '#description' => t('<strong>Opt-in:</strong> Users must sign up to recieve messages.<br/><strong>Opt-out: </strong> Users are automatically signed up but may unsubscribe.<br/><strong>Required: </strong> Users will remain on the book as long as they have an account and cannot unsubscribe.')
  );

  $element['doublein'] = array(
    '#type' => 'checkbox',
    '#title' => t('Require subscribers to Double Opt-in'),
    '#default_value' => $saved_book->doublein,
    '#description' => t('New subscribers will be sent a link with an email they must follow to confirm their subscription.'),
  );


  $datafields = dotmailer_listcontactdatalabels($address_book->ID);
  if ($datafields) {
    // Merge var fieldset
    $element['datafields'] = array(
      '#type' => 'fieldset',
      '#title' => t('Merge Variables'),
      '#description' => t('Select Drupal user variables to send to dotmailer as Merge Variables. Available Drupal variables are any Profile or Token variables for the given user. For more information on Merge Variables, see the !doc', array('!doc' => l(t('dotmailer Documentation'), 'http://server.iad. liveperson.net/hc/s-31286565/cmd/kbresource/kb-8214439208090042855/view_question!PAGETYPE?sq=merge%2bvariables&sf=101113&sg=0&st=188569&documentid=143258&action=view'))),
      '#tree' => TRUE,
    );
    $data_options = dotmailer_get_datafields_keys();
    foreach ($datafields->ContactDataLabel as $field) {
      $element['datafields'][$field->Name] = array(
        '#type' => 'select',
        '#title' => $field->Name,
        '#options' => $data_options,
        '#default_value' => $saved_book->datafields[$field->Name]
      );
    }
  }

  return $element;
}

/**
 * Provides a settings form for the case where there is no account set up yet.
 */
function _dotmailer_admin_settings_no_account(&$form, &$form_state) {
  // Show the username and password form so we can authenticate the user
  $form['dotmailer_account_info']['dotmailer_username'] = array(
    '#type' => 'textfield',
    '#title' => t('Dotmailer API username'),
    '#required' => TRUE,
    '#default_value' => variable_get('dotmailer_username', ''),
    '#description' => t('The username used to connect to the dotmailer API.')
  );
  $form['dotmailer_account_info']['dotmailer_password'] = array(
    '#type' => 'textfield',
    '#title' => t('Dotmailer API password'),
    '#required' => TRUE,
    '#default_value' => variable_get('dotmailer_password', ''),
    '#description' => t('The password used to connect to the dotmailer API.')
  );
  return $form;
}

/**
 * Submit the admin settings and serialize the saved books into objects
 *
 * @param <type> $form
 * @param <type> $form_state
 */
function dotmailer_admin_settings_validate($form, &$form_state) {
  // No books selected or first time here
  if (empty($form_state['values']['dotmailer_books'])) {
    return;
  }

	// Set active lists if values submitted
	if ($form_state['values']['dotmailer_active_lists']) {
		variable_set('dotmailer_active_lists', $form_state['values']['dotmailer_active_lists']);
	}

  variable_set('dotmailer_user_register', $form_state['values']['dotmailer_user_register']);
  variable_set('dotmailer_user_edit', $form_state['values']['dotmailer_user_edit']);

  // Loop through each book and create an array to serialize with,
  // saves storing lots of individual values per book
  $books = array();
  $sorted_books = array();

  foreach ($form_state['values']['dotmailer_books'] as $form_book) {
    $book = dotmailer_get_saved_book($form_book['book_id']);
    $book->Id = $form_book['book_id'];
    $book->Name = $form_book['name'];
    $book->title = $form_book['title'];
    $book->weight = $form_book['weight'];
    $book->external_url = $form_book['external_url'];
    $book->roles = array_filter((array)$form_book['roles']);
    $book->description = $form_book['description'];
    $book->booktype = $form_book['booktype'];
    $book->doublein = $form_book['doublein'];
    $book->datafields = $form_book['datafields'];
    $books[$form_book['weight'] .' '. $book->Name .' '. $book->Id] = $book;
  }
  ksort($books);
  foreach($books AS $this_book) {
    $sorted_books[$this_book->Id] = $this_book;
  }

  // Remove books from the form_state
  unset($form_state['values']['dotmailer_books']);
  variable_set('dotmailer_books', $sorted_books);
}